/*
Copyright (C) 2009 - Paolo Pagano and Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file DAQEvent.cxx
 *\author Paolo Pagano
 *\brief  DAQEvent functions development
 */

#include "DAQEvent.h"

ClassImp(DAQEvent)

TClonesArray *DAQEvent::fgDAQSDatas = 0;

DAQEvent::DAQEvent() 
{
	// Create an DAQSData object.
	// When the constructor is invoked for the first time, the class static
	// variable fgDAQSDatas is 0 and the TClonesArray fgDAQSDatas is created.
	printf("Constructor built\n");
	if (!fgDAQSDatas) 
		fgDAQSDatas = new TClonesArray("DAQSData", 1000);

	fDAQSDatas = fgDAQSDatas;
	fNDAQSDatas = 0;
	fDAQSDataName = 0;
	
	Int_t i0;
	for (i0 = 0; i0 <20; i0++) 
		fType[i0] = 0;
}

DAQEvent::~DAQEvent()
{
	Clear();
	if (fDAQSDataName) 
		delete [] fDAQSDataName;
}

//______________________________________________________________________________
void DAQEvent::Build(Int_t ev, std::vector< std::vector<SData> >* data) {
	char etype[20];

	Int_t ObjectNumber = TProcessID::GetObjectCount();
	Clear();

	Int_t nch = 15;
	if (ev >= 100)   
		nch += 3;
	if (ev >= 10000) 
		nch += 3;

	if (fDAQSDataName) 
		delete [] fDAQSDataName;

	fDAQSDataName = new char[nch];
	sprintf(fDAQSDataName,"DAQSData%d_Run%d",ev,200);
	sprintf(etype,"type%d",ev%5);
	SetType(etype);
	fNDAQSDatas = 0;

	//  Create and Fill the DAQSData objects
	for(unsigned int i = 0; i < data->size(); i++)
		AddDAQSData((*data)[i]);

	//Restore Object count 
	//To save space in the table keeping track of all referenced objects
	//we assume that our events do not address each other. We reset the 
	//object count to what it was at the beginning of the event.
	TProcessID::SetObjectCount(ObjectNumber);
}  

//______________________________________________________________________________
DAQSData *DAQEvent::AddDAQSData(std::vector<SData> sdata_)
{
	// Add a new track to the list of tracks for this event.
	// To avoid calling the very time consuming operator new for each track,
	// the standard but not well know C++ operator "new with placement"
	// is called. If tracks[i] is 0, a new DAQSData object will be created
	// otherwise the previous DAQSData[i] will be overwritten.

	TClonesArray &DAQSDatas = *fDAQSDatas;
	DAQSData *myevent = new(DAQSDatas[fNDAQSDatas++]) DAQSData(sdata_);
	fLastEvent = myevent;
	return myevent;  
}

//______________________________________________________________________________
void DAQEvent::Clear(Option_t * /*option*/)
{
	fDAQSDatas->Clear("C"); //will also call DAQSData::Clear
}

//______________________________________________________________________________
void DAQEvent::Reset(Option_t * /*option*/)
{
	// Static function to reset all static objects for this event
	// fgDAQSDatas->Delete(option);

	delete fgDAQSDatas; fgDAQSDatas = 0;
}

//______________________________________________________________________________

