/*
Copyright (C) 2009 - Paolo Pagano and Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 * @file   DAQPayload.h
 * @author Paolo Pagano and Claudio Salvadori
 * @date   22-08-2009
 * 
 * @brief  This file contains the stucture to store the data
 * according with xml file
 */

#ifndef DAQPAYLOAD_H
#define DAQPAYLOAD_H

#ifdef DAQSOCKET1
#define MESSAGE_BUFFER		4
#endif

#ifdef DAQSOCKET2
#define MESSAGE_BUFFER		4
#endif

#ifdef DAQIMAGE
#define IMAGE_BUFFER		19200
#endif

#ifdef DAQSNIFFER
#define PAYLOAD_BUFFER		128
#endif

#ifdef DAQTIMERAVR
#define NUMBER_OF_PACKET	4
#endif

#ifdef DAQTIMERAVR_EE
#define NUMBER_OF_PACKET	1
#endif

#ifdef DAQSTEREO
#define STEREO_BUFFER		5520
#define STEREO_WIDTH		80
#define STEREO_HEIGHT		69
#endif


/**
*\brief Structure that describe the packet payload according with xml file
*\warning This structure has to be modified everytime the xml file payload description is changed
*/

struct DAQPayload{

#ifdef DAQSOCKET1
	Char_t  	message_[MESSAGE_BUFFER];   	/**<Message arrived*/
	UChar_t  	separator_;			/**<Separator value*/
	UInt_t  	counter_;			/**<Counter value*/
#endif

#ifdef DAQSOCKET2
	Char_t  	message_[MESSAGE_BUFFER];   	/**<Message arrived*/
	UChar_t  	separator1_;			/**<Separator1 value*/
	UInt_t  	random1_;			/**<Random1 value*/
	UChar_t  	separator2_;			/**<Separator2 value*/
	UInt_t  	random2_;			/**<Random2 value*/
#endif

#ifdef DAQIMAGE
	UShort_t  	width_;				/**<Image width*/
	UShort_t  	height_;			/**<Image height*/
	ULong_t  	duration_;			/**<Image duration*/
	UShort_t  	lost_;				/**<Frame lost (1 true, 0 false)*/
	UShort_t  	ymean_;				/**<Luminance mean value*/
	UChar_t  	image_[IMAGE_BUFFER];		/**<Raw image*/
#endif

#ifdef DAQSNIFFER
	UInt_t  	timestamp_;			/**<Beacon timestamp*/
	UChar_t  	payload_[PAYLOAD_BUFFER];	/**<Packet payload*/
#endif

#ifdef DAQTIMERAVR
	UShort_t  	cycles_[NUMBER_OF_PACKET];	/**<Number of 8 bits timer overflow*/
	UChar_t  	tn_[NUMBER_OF_PACKET];   	/**<Value of timer register*/
	UShort_t  	seqnum_;  			/**<Packet number*/
	UShort_t	pkt_length_;			/**<Packet length in bytes*/
#endif

#ifdef DAQTIMERUW_EE
	UChar_t  	seqnum_;  			/**<Packet number*/
	UInt_t  	time_;				/**<End-to-End duration 
							(from application layer to application layer)*/
	UShort_t  	length_;   			/**<Packet lenght*/
#endif

#ifdef DAQTIMERUW
	UInt_t  	time_;				/**<Stack duration 
							(from application layer to physical layer)*/
	UShort_t  	length_;   			/**<Packet lenght*/
#endif

#ifdef DAQTRACKING
	UShort_t 	position_[2];			/**<Centroid position in a 2D space*/
#endif

#ifdef DAQMIRTES
	UShort_t  	counter_;			/**<Samples counter*/
	UShort_t  	minimum_;   			/**<Minimum luminosity value*/
	UShort_t  	maximum_;			/**<Maximum luminosity value*/
	Float_t  	average_;			/**<Average of the luminosity value*/
#endif

#ifdef DAQSTEREO
	UChar_t  	image_dx_[STEREO_BUFFER];	/**<Raw image (right side)*/
	UChar_t  	image_sx_[STEREO_BUFFER];	/**<Raw image (left side)*/
	UChar_t  	image_stereo_[STEREO_BUFFER];	/**<Raw image (stereo image)*/
#endif
};

#endif
