/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/**
 *\file IRenderer.cpp
 *\brief IRenderer class not-virtual method development.*/

#include <exception>
#include <iostream>
#include "IRenderer.h"

std::map<std::string, IRenderer *(*)(const void*)> *IRenderer::Factory::creators;
std::vector<IRenderer*> *IRenderer::Factory::renderers;
std::vector<std::string> *IRenderer::Factory::nameV_;

class FactoryException : public std::exception {
public:
        FactoryException(const char *msg) throw() : errMessage(msg) { } ;
        ~FactoryException(void) throw() { } ;
        const char *what(void) const throw() { return errMessage.c_str(); };
private:
       	std::string errMessage;
        FactoryException(void) throw();
};

IRenderer::Factory::Factory(std::string semantic, IRenderer *(*creator)(const void* node))
{
	if(!creators)
		creators = new std::map<std::string, IRenderer *(*)(const void*)>();	
			
	std::map<std::string, IRenderer *(*)(const void*)>::iterator iter = creators->find(semantic);
	
	std::cerr << "Semantic: " << semantic << std::endl;
	
    	if(iter != creators->end())
    		throw FactoryException("Semantic ID alredy registered!");
    		
    	creators->insert(std::make_pair(semantic, creator));
}

void IRenderer::create(std::string semantic, const void* node, var_type vType)
{	
	std::map<std::string, IRenderer *(*)(const void*)>::iterator iter = Factory::creators->find(semantic);
    	if(iter == Factory::creators->end())
    		throw FactoryException("Semantic ID is not registered!");
    		
    	if(!Factory::renderers)
		Factory::renderers = new std::vector<IRenderer*>();
    		
    	Factory::renderers->push_back(Factory::creators->at(semantic)(node));
    	
    	Factory::renderers->at(Factory::renderers->size() - 1)->setType(vType);
    	
}

IRenderer *IRenderer::getRenderer(unsigned int i){
	return Factory::renderers->at(i);
}

void IRenderer::checkName(std::string name){
	if(!Factory::nameV_)
		Factory::nameV_ = new std::vector<std::string>();
	
	//Check if the file name exists yet
	if(Factory::nameV_->size() != 0){
		for(unsigned int i =0; i < Factory::nameV_->size(); i++){
			if(Factory::nameV_->at(i) == name)
				throw FactoryException("Renderer name exists yet");
		}
	}
	Factory::nameV_->push_back(name);
}

void IRenderer::endCheck(void){
	delete Factory::nameV_;
}
