/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file Show_image.cpp
 *\brief  Image rendering function development
 */

#include "Show_image.h"
#include <myUtils.h>

Fl_Double_Window *screen_wnd_SX = (Fl_Double_Window *) 0;
Fl_Double_Window *screen_wnd_DX = (Fl_Double_Window *) 0;
Fl_Double_Window *screen_wnd_Disp = (Fl_Double_Window *) 0;

Fl_Box *screen_img_SX = (Fl_Box *) 0;
Fl_Box *screen_img_DX = (Fl_Box *) 0;
Fl_Box *screen_img_Disp = (Fl_Box *) 0;

static Fl_RGB_Image *m_image_SX;
static Fl_RGB_Image *m_image_DX;
static Fl_RGB_Image *m_image_Disp;

Fl_Double_Window *screen_wnd = (Fl_Double_Window *) 0;
Fl_Box *screen_img = (Fl_Box *) 0;
static Fl_RGB_Image *m_image;

Fl_Double_Window *screen_wnd_black = (Fl_Double_Window *) 0;
Fl_Box *screen_img_black = (Fl_Box *) 0;
static Fl_RGB_Image *m_image_black;
static unsigned char *image_black;

/** \todo Add parameter to dimension the size of the windows*/
void initRenderer(unsigned int w, unsigned int h){
	if((screen_wnd == (Fl_Double_Window *) 0) && (screen_img == (Fl_Box *) 0)){
		Fl::visual(FL_RGB);
		{
			screen_wnd = new Fl_Double_Window(w + 30 , h + 30, "Screen");
			screen_wnd->box(FL_BORDER_BOX);
			{
				//Attention!! m_box has to have only this setting!!!
				screen_img = new Fl_Box(15, 15, w, h);
				screen_img->box(FL_FLAT_BOX);
			}
			screen_wnd->end();
		}
		
		screen_wnd->show();
		Fl::check();
	}
	return;
}

void renderImage(unsigned char *image, unsigned int w, unsigned int h, unsigned int channel){
	fflush(stdout);
	if(m_image) delete m_image;

	m_image = new Fl_RGB_Image(image, w, h,channel);
	screen_img->image(m_image);
	screen_img->redraw();
	Fl::check();
	return;
}

/** \todo Add parameter to dimension the size of the windows*/
void initRenderer_black(unsigned int w, unsigned int h){
	if((screen_wnd_black == (Fl_Double_Window *) 0) && (screen_img_black == (Fl_Box *) 0)){
		Fl::visual(FL_RGB);
		{
			screen_wnd_black = new Fl_Double_Window(w + 30 , h + 30, "Screen");
			screen_wnd_black->box(FL_BORDER_BOX);
			{
				//Attention!! m_box has to have only this setting!!!
				screen_img_black = new Fl_Box(15, 15, w, h);
				screen_img_black->box(FL_FLAT_BOX);
			}
			screen_wnd_black->end();
		}
		
		screen_wnd_black->show();
		Fl::check();
	}
	
	image_black = new unsigned char[w * h];
	//Fill image with 0x00
	for(unsigned int j = 0; j < w * h; j++){
		*(image_black + j) =	0x00;
	}

	return;
}

void renderImage_black(unsigned int w, unsigned int h, unsigned int x, unsigned int y){
	unsigned char image_black_24[w * h * 3];

	fflush(stdout);

	//Convert from gray8 to rgb24
	my_gray8_to_rgb24(image_black, image_black_24, w, h);

	//Paint the point
	my_draw_point(image_black_24, w, h, x, y, MY_RED, 6, 3);

	m_image_black = new Fl_RGB_Image(image_black_24, w, h, 3);
	
	screen_img_black->image(m_image_black);
	screen_img_black->redraw();
	Fl::check();
	return;
}

void initRendererSX(unsigned int w, unsigned int h){
	if((screen_wnd_SX == (Fl_Double_Window *) 0) && (screen_img_SX == (Fl_Box *) 0)){
		Fl::visual(FL_RGB);
		{
			screen_wnd_SX = new Fl_Double_Window(w + 30 , h + 30, "Left");
			screen_wnd_SX->box(FL_BORDER_BOX);
			{
				//Attention!! m_box has to have only this setting!!!
				screen_img_SX = new Fl_Box(15, 15,  w, h);
				screen_img_SX->box(FL_FLAT_BOX);
			}
			screen_wnd_SX->end();
		}
		
		screen_wnd_SX->show();
		Fl::check();
	}
	return;
}


void renderImageSX(unsigned char *image, unsigned int w, unsigned int h, unsigned int channel){
	fflush(stdout);
	if(m_image_SX) delete m_image_SX;

	m_image_SX = new Fl_RGB_Image(image, w, h,channel);
	screen_img_SX->image(m_image_SX);
	screen_img_SX->redraw();
	Fl::check();
	return;

}

void initRendererDX(unsigned int w, unsigned int h){
	if((screen_wnd_DX == (Fl_Double_Window *) 0) && (screen_img_DX == (Fl_Box *) 0)){
		Fl::visual(FL_RGB);
		{
			screen_wnd_DX = new Fl_Double_Window(w + 30 , h + 30, "Right");
			screen_wnd_DX->box(FL_BORDER_BOX);
			{
				//Attention!! m_box has to have only this setting!!!
				screen_img_DX = new Fl_Box(15, 15,  w, h);
				screen_img_DX->box(FL_FLAT_BOX);
			}
			screen_wnd_DX->end();
		}
		
		screen_wnd_DX->show();
		Fl::check();
	}
	return;
}


void renderImageDX(unsigned char *image, unsigned int w, unsigned int h, unsigned int channel){
	fflush(stdout);
	if(m_image_DX) delete m_image_DX;

	m_image_DX = new Fl_RGB_Image(image, w, h,channel);
	screen_img_DX->image(m_image_DX);
	screen_img_DX->redraw();
	Fl::check();
	return;

}


void initRendererDisp(unsigned int w, unsigned int h){
	if((screen_wnd_Disp == (Fl_Double_Window *) 0) && (screen_img_Disp == (Fl_Box *) 0)){
		Fl::visual(FL_RGB);
		{
			screen_wnd_Disp = new Fl_Double_Window(w + 30 , h + 30, "Disp");
			screen_wnd_Disp->box(FL_BORDER_BOX);
			{
				//Attention!! m_box has to have only this setting!!!
				screen_img_Disp = new Fl_Box(15, 15,  w, h);
				screen_img_Disp->box(FL_FLAT_BOX);
			}
			screen_wnd_Disp->end();
		}
		
		screen_wnd_Disp->show();
		Fl::check();
	}
	return;
}


void renderImageDisp(unsigned char *image, unsigned int w, unsigned int h, unsigned int channel){
	fflush(stdout);
	if(m_image_Disp) delete m_image_Disp;

	m_image_Disp = new Fl_RGB_Image(image, w, h,channel);
	screen_img_Disp->image(m_image_Disp);
	screen_img_Disp->redraw();
	Fl::check();
	return;

}

