/*
Copyright (C) 2007 - Nastasi Christian
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file myUtils.cpp
 *\brief  Vision algorithms development functions
 */

#include "myUtils.h"

typedef struct
{
	struct jpeg_source_mgr 	pub;   				
	unsigned char* 			buffer;			
	unsigned int   			bufsize;		
} myJPEG_source_mgr;

typedef myJPEG_source_mgr* myJPEG_src_ptr; 

/*-------------- IMAGE OBJECT DRAWING ----------------------*/
void my_draw_point(unsigned char *im, unsigned int width, unsigned int height, unsigned int x, unsigned int y, My_draw_color color, int space, int bold)
{
	/* Work on a RGB24 image */
	
	unsigned int i,j, max_i, max_j, min_j;
	unsigned char red, green, blue;

	switch(color)
	{
	  case MY_RED:
		red   = 255;
		green = 0;
		blue  = 0;
		break;

	  case MY_GREEN:
		red   = 0;
		green = 255;
		blue  = 0;
		break;

	  case MY_BLUE:
		red   = 0;
		green = 0;
		blue  = 255;
		break;
	}
	

	if((int)y-space < 0) i=0;
	else i=y-space;
	if((int)x-space < 0) min_j=0;
	else min_j=x-space;

	if(y+space > height) max_i=height;
	else max_i=y+space;
	if(x+space > width) max_j=width;
	else max_j=x+space;

	for(; i<max_i; i++){
		for(j=min_j; j<max_j; j++){
			if(((int)i>(int)y-bold) && (i<y+bold)){
				*(im+(i*width*3)+(j*3))= red;
				*(im+(i*width*3)+(j*3)+1)= green;
				*(im+(i*width*3)+(j*3)+2)= blue;
			}
			else if(((int)j>(int)x-bold) && (j<x+bold)){
				*(im+(i*width*3)+(j*3))= red;
				*(im+(i*width*3)+(j*3)+1)= green;
				*(im+(i*width*3)+(j*3)+2)= blue;
			}
		}
	}
}

void my_draw_rect(	unsigned char *im, unsigned int width, unsigned int height, 
			unsigned int tlx, unsigned int tly, unsigned int brx, unsigned int bry, My_draw_color color, int bold)
{
	/* Work on a RGB24 image */
	
	unsigned int i,j;
	unsigned char red, green, blue;

	switch(color)
	{
	  case MY_RED:
		red   = 255;
		green = 0;
		blue  = 0;
		break;

	  case MY_GREEN:
		red   = 0;
		green = 255;
		blue  = 0;
		break;

	  case MY_BLUE:
		red   = 0;
		green = 0;
		blue  = 255;
		break;
	}

	for(j=tly-bold; j< tly+bold; j++){
		for(i=tlx-bold; i< brx+bold; i++){
			if((i>0) &&  (i<width) && (j>0) && (j<height)){
				*(im+(j*width*3)+(i*3))   = red;
				*(im+(j*width*3)+(i*3)+1) = green;
				*(im+(j*width*3)+(i*3)+2) = blue;
			}
		}
	}
	for(j=bry-bold; j< bry+bold; j++){
		for(i=tlx-bold; i< brx+bold; i++){
			if((i>0) &&  (i< width) && (j>0) && (j<height)){
				*(im+(j*width*3)+(i*3))   = red;
				*(im+(j*width*3)+(i*3)+1) = green;
				*(im+(j*width*3)+(i*3)+2) = blue;
			}
		}
	}
	for(i=tlx-bold; i< tlx+bold; i++){
		for(j=tly-bold; j< bry+bold; j++){
			if((i>0) &&  (i< width) && (j>0) && (j<height)){
				*(im+(j*width*3)+(i*3))   = red;
				*(im+(j*width*3)+(i*3)+1) = green;
				*(im+(j*width*3)+(i*3)+2) = blue;
			}
		}
	}
	for(i=brx-bold; i< brx+bold; i++){
		for(j=tly-bold; j< bry+bold; j++){
			if((i>0) &&  (i< width) && (j>0) && (j<height)){
				*(im+(j*width*3)+(i*3))   = red;
				*(im+(j*width*3)+(i*3)+1) = green;
				*(im+(j*width*3)+(i*3)+2) = blue;
			}
		}
	}
}

/*-------------- COLOR CONVERSIONS ----------------------*/


void my_gray8_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height)
{
	unsigned long int i;

	for(i=0; i<width*height; i++){
		*dest = *src;
		*(dest+1) = *src;
		*(dest+2) = *src;
		dest+=3;
		src++;
	}
}

void my_rgb16_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height)
{
	/* Assuming ... 16 bit = 5(red) - 6(green) - 5(blue) */
	
	unsigned long int i;

	for(i=0; i<width*height; i++){
		*(dest++) = (*src) & 0xF8 ; 
		*(dest++) = ((*src) << 5) + (((*(src+1))>>3) & 0X1C) ; 
		*(dest++) = (*(src+1)) << 3; 
		src+=2; 
	}
}

void my_rgb12_to_rgb24(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height)
{
	/* Assuming ... 12 bit = 4(red) - 4(green) - 4(blue) */
	
	unsigned long int i;

	for(i=0; i<width*height; i++){
		if(i%2){
			*(dest++) = (*src) & 0xF0; 
			*(dest++) = (*(src++)) << 4; 
		}
		else{
			*(dest++) = (*src) & 0xF0; 
			*(dest++) = (*(src++)) << 4; 
			*(dest++) = (*src) & 0xF0; 
			*(dest++) = (*(src++)) << 4; 
		}
	}
}


void my_gray4_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height)
{
	/* Assuming ... 4 bit single channel */

	unsigned long int i;

	for(i=0; i<width*height; i++){
		if(i%2)
			*(dest++) = (*(src++)) << 4; 
		else
			*(dest++) = (*src) & 0xF0; 
	}
}

void my_gray2_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height)
{
	/* Assuming ... 4 bit single channel */

	unsigned long int i;

	for(i=0; i<width*height; i++){
		if(i%4==0)
			*(dest++) = (*src) & 0xC0; 
		else if(i%4==1)
			*(dest++) = ((*src) & 0x30) << 2; 
		else if(i%4==2)
			*(dest++) = ((*src) & 0x0C) << 4; 
		else
			*(dest++) = (*(src++)) << 6; 
	}
}

void my_bwBin_to_gray8(unsigned char *src, unsigned char *dest, unsigned int width, unsigned int height)
{
	unsigned long int i;
	unsigned char mask=128;
	

	for(i=0; i<width*height; i++) {
		if((*src & mask)!=0)
			*dest=255;
		else
			*dest=0;

		mask >>= 1;
		if(mask==0){
			mask=128;
			src++;
		}
		dest++;
	}
}


/*-------------- JPEG DECOMPRESSION ----------------------*/


/*
 Initialize source --- Nothing to do
 */
METHODDEF(void)
init_source (j_decompress_ptr cinfo) {}

/*
 Fill the input buffer --- called whenever buffer is emptied.
 */
METHODDEF(boolean)
fill_input_buffer (j_decompress_ptr cinfo)
{
	myJPEG_src_ptr src = (myJPEG_src_ptr) cinfo->src;

	src->pub.next_input_byte = src->buffer;
	src->pub.bytes_in_buffer = src->bufsize;

	return TRUE;
}

/*
 Skip data --- used to skip over a potentially large amount of
 uninteresting data.
 */
METHODDEF(void)
skip_input_data (j_decompress_ptr cinfo, long num_bytes)
{
	myJPEG_src_ptr src = (myJPEG_src_ptr) cinfo->src;

	/* just move the ptr */
	src->pub.next_input_byte += num_bytes;
	src->pub.bytes_in_buffer -= num_bytes;
}

/*
  Terminate source --- called by jpeg_finish_decompress
 */
METHODDEF(void)
term_source (j_decompress_ptr cinfo) {}

/*
 Prepare for input from a memory buffer.
 */
GLOBAL(void)
jpeg_mem_src (j_decompress_ptr cinfo, unsigned char* buffer, unsigned int bufsize)
{
	myJPEG_src_ptr src;

	if (cinfo->src == NULL) {
		cinfo->src = (struct jpeg_source_mgr *)
		(*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,  sizeof(myJPEG_source_mgr));
  	}

	src = (myJPEG_src_ptr) cinfo->src;
	src->pub.init_source = init_source;
	src->pub.fill_input_buffer = fill_input_buffer;
	src->pub.skip_input_data = skip_input_data;
	src->pub.resync_to_restart = jpeg_resync_to_restart; 
	src->pub.term_source = term_source;
	src->pub.bytes_in_buffer = 0; 
	src->pub.next_input_byte = NULL; 

	src->buffer = buffer;
	src->bufsize = bufsize;
}

bool my_JPEG_decompress(unsigned char* srcBuffer, int size, unsigned char* dstBuffer, int& width, int& height, int& bpp)
{
	if (!dstBuffer)
		return false;

	struct jpeg_error_mgr jerr;
	struct jpeg_decompress_struct cinfo;

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

	jpeg_mem_src (&cinfo, srcBuffer, size);

	jpeg_read_header(&cinfo,true);

	cinfo.quantize_colors      = (boolean)FALSE;
	cinfo.out_color_space      = JCS_RGB;
	cinfo.out_color_components = 3;
	cinfo.output_components    = 3;

	jpeg_start_decompress(&cinfo);
	
	width = cinfo.output_width;
	height = cinfo.output_height;
	bpp = cinfo.num_components;
	unsigned char* rowptr[1];
	while (cinfo.output_scanline < cinfo.output_height)
	{
		rowptr[0] = &dstBuffer[cinfo.output_scanline*cinfo.output_width*cinfo.num_components];
		if (rowptr[0] == 0) {
			jpeg_finish_decompress(&cinfo);
			jpeg_destroy_decompress(&cinfo);
			return false;
		}
		if (jpeg_read_scanlines(&cinfo, rowptr, 1) != 1) {
			jpeg_finish_decompress(&cinfo);
			jpeg_destroy_decompress(&cinfo);
			return false;
		}
	}
	jpeg_finish_decompress(&cinfo);
	jpeg_destroy_decompress(&cinfo);

	return true;
}



